# Copyright 2017-2020 The FIAAS Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from .common import ObjectMeta
from ..base import Model
from ..fields import Field, ListField


class RoleRef(Model):
    apiGroup = Field(str)
    kind = Field(str)
    name = Field(str)


class Subject(Model):
    kind = Field(str)
    name = Field(str)
    apiGroup = Field(str)
    namespace = Field(str)


class RoleBinding(Model):
    class Meta:
        list_url = "/apis/rbac.authorization.k8s.io/v1/rolebindings"
        url_template = "/apis/rbac.authorization.k8s.io/v1/namespaces/{namespace}/rolebindings/{name}"

    metadata = Field(ObjectMeta)
    roleRef = Field(RoleRef)
    subjects = ListField(Subject)



#!/usr/bin/env python
# -*- coding: utf-8

# Copyright 2017-2019 The FIAAS Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import mock
import pytest

from k8s.client import NotFound
from k8s.models.common import ObjectMeta
from k8s.models.apiextensions_v1_custom_resource_definition import (
    CustomResourceConversion, CustomResourceDefinition, CustomResourceDefinitionNames,
    CustomResourceDefinitionSpec, CustomResourceDefinitionVersion, JSONSchemaProps)

NAME = "my-name"


# pylint: disable=R0201
@pytest.mark.usefixtures("k8s_config")
class TestCustomResourceDefinition(object):
    def test_create_blank(self):
        object_meta = ObjectMeta(name=NAME, labels={"test": "true"})
        crd = CustomResourceDefinition(metadata=object_meta)
        assert crd.as_dict()[u"metadata"][u"name"] == NAME

    def test_created_if_not_exists(self, post, api_get):
        api_get.side_effect = NotFound()
        crd = _create_default_crd()
        call_params = crd.as_dict()
        post.return_value.json.return_value = call_params

        assert crd._new
        crd.save()
        assert not crd._new

        pytest.helpers.assert_any_call(post, _uri(), call_params)

    def test_updated_if_exists(self, get, put):
        mock_response = _create_mock_response()
        get.return_value = mock_response
        crd = _create_default_crd()

        from_api = CustomResourceDefinition.get_or_create(metadata=crd.metadata, spec=crd.spec)
        assert not from_api._new
        call_params = from_api.as_dict()
        put.return_value.json.return_value = call_params

        from_api.save()
        pytest.helpers.assert_any_call(put, _uri(NAME), call_params)

    def test_nested_json_schema_props(self, put):
        crd_dict = _json_schema_props_crd_dict()
        crd = CustomResourceDefinition.from_dict(crd_dict)

        call_params = crd.as_dict()
        put.return_value.json.return_value = call_params

        assert isinstance(crd.spec.versions[0].schema.openAPIV3Schema, JSONSchemaProps)
        assert isinstance(crd.spec.versions[0].schema.openAPIV3Schema._not, JSONSchemaProps)

        crd.save()
        pytest.helpers.assert_any_call(put, _uri(NAME), call_params)

    def test_json_schema_props_any_field(self, put):
        for default in [42, "string", True, [], {}]:
            crd_dict = _json_schema_props_crd_dict(default)
            crd = CustomResourceDefinition.from_dict(crd_dict)

            call_params = crd.as_dict()
            put.return_value.json.return_value = call_params

            assert isinstance(crd.spec.versions[0].schema.openAPIV3Schema.default, type(default))

            crd.save()
            pytest.helpers.assert_any_call(put, _uri(NAME), call_params)


def _json_schema_props_crd_dict(default=None):
    return {
        "apiVersion": "apiextensions.k8s.io/v1",
        "kind": "CustomResourceDefinition",
        "metadata": {
            "creationTimestamp": "2019-11-23T13:43:42Z",
            "generation": 7,
            "labels": {
                "test": "true"
            },
            "name": NAME,
            "resourceVersion": "96758807",
            "selfLink": _uri(NAME),
            "uid": "d8f1ba26-b182-11e6-a364-fa163ea2a9c4"
        },
        "spec": {
            "conversion": {"strategy": "None"},
            "group": "example.com",
            "names": {
                "kind": "MyCustomResource",
                "plural": "mycustomresources"
            },
            "scope": "namespaced",
            "versions": [{
                "name": "v1",
                "served": True,
                "schema": {
                    "openAPIV3Schema": {
                        "type": "object",
                        "properties": {
                            "spec": {
                                "type": "object",
                                "properties": {
                                    "cronSpec": {
                                        "type": "string"
                                    },
                                    "image": {
                                        "type": "string"
                                    },
                                    "replicas": {
                                        "type": "integer"
                                    }
                                }
                            }
                        },
                        "not": {
                            "type": "integer",
                        },
                        "default": default
                    }
                }
            }]
        }
    }


def _create_mock_response():
    mock_response = mock.Mock()
    mock_response.json.return_value = {
        "apiVersion": "apiextensions.k8s.io/v1",
        "kind": "CustomResourceDefinition",
        "metadata": {
            "creationTimestamp": "2019-11-23T13:43:42Z",
            "generation": 7,
            "labels": {
                "test": "true"
            },
            "name": NAME,
            "resourceVersion": "96758807",
            "selfLink": _uri(NAME),
            "uid": "d8f1ba26-b182-11e6-a364-fa163ea2a9c4"
        },
        "spec": {
            "conversion": {"strategy": "None"},
            "group": "example.com",
            "names": {
                "kind": "MyCustomResource",
                "plural": "mycustomresources"
            },
            "scope": "namespaced",
            "versions": [{
                "name": "v1",
                "served": True
            }]
        }
    }
    return mock_response


def _create_default_crd():
    object_meta = ObjectMeta(name=NAME, labels={"test": "true"})
    spec = CustomResourceDefinitionSpec(
        conversion=CustomResourceConversion(strategy="None"),
        group="example.com",
        names=CustomResourceDefinitionNames(kind="MyCustomResource", plural="mycustomresources"),
        scope="Namespaced",
        versions=[CustomResourceDefinitionVersion(name="v42", served=True)]
    )
    crd = CustomResourceDefinition(metadata=object_meta, spec=spec)
    return crd


def _uri(name=""):
    return "/apis/apiextensions.k8s.io/v1/customresourcedefinitions/{name}".format(name=name)



#!/usr/bin/env python
# -*- coding: utf-8

# Copyright 2017-2019 The FIAAS Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime

from ..base import Model
from ..fields import Field, ReadOnlyField, ListField


class OwnerReference(Model):
    apiVersion = Field(str)
    blockOwnerDeletion = Field(bool)
    controller = Field(bool)
    kind = Field(str)
    name = Field(str)
    uid = Field(str)


class ObjectMeta(Model):
    annotations = Field(dict)
    creationTimestamp = ReadOnlyField(datetime.datetime)
    deletionGracePeriodSeconds = ReadOnlyField(int)
    deletionTimestamp = ReadOnlyField(datetime.datetime)
    finalizers = ListField(str)
    generateName = Field(str)
    generation = ReadOnlyField(int)
    labels = Field(dict)
    name = Field(str)
    namespace = Field(str, "default")
    ownerReferences = ListField(OwnerReference)
    resourceVersion = ReadOnlyField(str)
    selfLink = ReadOnlyField(str)
    uid = ReadOnlyField(str)


class TypedLocalObjectReference(Model):
    apiGroup = Field(str)
    kind = Field(str)
    name = Field(str)


class ObjectReference(Model):
    apiVersion = Field(str)
    fieldPath = Field(str)
    kind = Field(str)
    name = Field(str)
    namespace = Field(str)
    resourceVersion = Field(str)
    uid = Field(str)


class Preconditions(Model):
    resourceVersion = Field(str)
    uid = Field(str)


class DeleteOptions(Model):
    apiVersion = Field(str)
    dryRun = ListField(str)
    gracePeriodSeconds = Field(int)
    kind = Field(str)
    preconditions = Field(Preconditions)
    propagationPolicy = Field(str)


class LocalObjectReference(Model):
    name = Field(str)



#!/usr/bin/env python
# -*- coding: utf-8

# Copyright 2017-2020 The FIAAS Authors
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#      http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import mock
import pytest

from k8s.client import NotFound
from k8s.models.common import ObjectMeta
from k8s.models.cronjob import CronJob, CronJobSpec
from k8s.models.job import JobSpec, JobTemplateSpec
from k8s.models.pod import Container, PodTemplateSpec, PodSpec


NAME = "my-name"
NAMESPACE = "my-namespace"


@pytest.mark.usefixtures("k8s_config")
class TestCronJobs(object):
    def test_create_blank_job(self):
        object_meta = ObjectMeta(name=NAME, namespace=NAMESPACE)
        cronjob = CronJob(metadata=object_meta)
        assert cronjob.as_dict()[u"metadata"][u"name"] == NAME

    def test_created_if_not_exists(self, post, api_get):
        api_get.side_effect = NotFound()
        cronjob = _create_default_cronjob()
        call_params = cronjob.as_dict()
        post.return_value.json.return_value = call_params

        assert cronjob._new
        cronjob.save()
        assert not cronjob._new

        pytest.helpers.assert_any_call(post, _uri(NAMESPACE), call_params)


def _create_default_cronjob():
    object_meta = ObjectMeta(name=NAME, namespace=NAMESPACE)
    container = Container(
        name="container",
        image="dummy_image"
    )
    pod_spec = PodSpec(containers=[container], serviceAccountName="default",
                       restartPolicy="OnFailure")
    pod_template_spec = PodTemplateSpec(metadata=object_meta, spec=pod_spec)
    job_spec = JobSpec(template=pod_template_spec)
    job_template_spec = JobTemplateSpec(metadata=object_meta, spec=job_spec)
    cronjob_spec = CronJobSpec(concurrencyPolicy="Allow", failedJobsHistoryLimit=0, jobTemplate=job_template_spec,
                               schedule="*/1 * * * *", startingDeadlineSeconds=5, successfulJobsHistoryLimit=100,
                               suspend=True)
    cronjob = CronJob(metadata=object_meta, spec=cronjob_spec)

    return cronjob


def _create_mock_response():
    mock_response = mock.Mock()
    mock_response.json.return_value = {
        u"metadata": {
            u"labels": {
                u"test": u"true"
            },
            u"namespace": u"my-namespace",
            u"name": u"my-name"
        },
        u"spec": {
            u"schedule": u"*/1 * * * *",
            u"jobTemplate": {
                u"spec": {
                    u"template": {
                        u"spec": {
                            u"serviceAccountName": u"default",
                            u"restartPolicy": u"Never",
                            u"volumes": [],
                            u"imagePullSecrets": [
                                {
                                    u"name": u"image_pull_secret"
                                }
                            ],
                            u"containers": [
                                {
                                    u"name": u"container",
                                    u"image": u"dummy_image",
                                    u"volumeMounts": [],
                                    u"env": [],
                                    u"imagePullPolicy": u"IfNotPresent",
                                }
                            ]
                        },
                        u"metadata": {
                            u"labels": {
                                u"test": u"true"
                            },
                            u"namespace": u"my-namespace",
                            u"name": u"my-name"
                        }
                    }
                }
            }
        }
    }
    return mock_response


def _uri(namespace, name=""):    
    return "/apis/batch/v1beta1/namespaces/{namespace}/cronjobs/{name}".format(name=name, namespace=namespace)



#!/usr/bin/env python
# -*- coding: utf-8

# Copyright 2017-2019 The FIAAS Authors
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#      http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from .common import ObjectMeta
from .pod import PodTemplateSpec
from ..base import Model
from ..fields import Field


class LabelSelector(Model):
    matchLabels = Field(dict)


class RollingUpdateDeployment(Model):
    maxUnavailable = Field(int, alt_type=str)
    maxSurge = Field(int, alt_type=str)


class DeploymentStrategy(Model):
    type = Field(str, "RollingUpdate")
    rollingUpdate = Field(RollingUpdateDeployment)


class DeploymentSpec(Model):
    replicas = Field(int, 1)
    selector = Field(LabelSelector)
    template = Field(PodTemplateSpec)
    strategy = Field(DeploymentStrategy)
    minReadySeconds = Field(str, alt_type=int)
    revisionHistoryLimit = Field(int)
    paused = Field(str)


class DeploymentStatus(Model):
    observedGeneration = Field(int)
    replicas = Field(int)
    updatedReplicas = Field(int)
    availableReplicas = Field(int)
    unavailableReplicas = Field(int)


class Deployment(Model):
    class Meta:
        list_url = "/apis/apps/v1/deployments"
        url_template = "/apis/apps/v1/namespaces/{namespace}/deployments/{name}"

    metadata = Field(ObjectMeta)
    spec = Field(DeploymentSpec)
    status = Field(DeploymentStatus)



#!/usr/bin/env python
# -*- coding: utf-8

# Copyright 2017-2019 The FIAAS Authors
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#      http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from .common import ObjectMeta
from ..base import Model
from ..fields import Field, ListField


class IngressBackend(Model):
    serviceName = Field(str)
    servicePort = Field(str)


class HTTPIngressPath(Model):
    path = Field(str)
    backend = Field(IngressBackend)


class HTTPIngressRuleValue(Model):
    paths = ListField(HTTPIngressPath)


class IngressRule(Model):
    host = Field(str)
    http = Field(HTTPIngressRuleValue)


class IngressTLS(Model):
    hosts = ListField(str)
    secretName = Field(str)


class IngressSpec(Model):
    backend = Field(IngressBackend)
    rules = ListField(IngressRule)
    tls = ListField(IngressTLS)


class Ingress(Model):
    class Meta:
        list_url = "/apis/extensions/v1beta1/ingresses"
        url_template = "/apis/extensions/v1beta1/namespaces/{namespace}/ingresses/{name}"
        watch_list_url = "/apis/extensions/v1beta1/watch/ingresses"

    metadata = Field(ObjectMeta)
    spec = Field(IngressSpec)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2017-2019 The FIAAS Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import pytest

from k8s.base import Model, Field, WatchEvent, Equality, Inequality, In, NotIn, Exists
from k8s.models.common import DeleteOptions, Preconditions


class Example(Model):
    class Meta:
        url_template = '/example'
        watch_list_url = '/watch/example'

    value = Field(int)


class TestWatchEvent(object):
    def test_watch_event_added(self):
        watch_event = WatchEvent({"type": "ADDED", "object": {"value": 42}}, Example)
        assert watch_event.type == WatchEvent.ADDED
        assert watch_event.object == Example(value=42)

    def test_watch_event_modified(self):
        watch_event = WatchEvent({"type": "MODIFIED", "object": {"value": 42}}, Example)
        assert watch_event.type == WatchEvent.MODIFIED
        assert watch_event.object == Example(value=42)

    def test_watch_event_deleted(self):
        watch_event = WatchEvent({"type": "DELETED", "object": {"value": 42}}, Example)
        assert watch_event.type == WatchEvent.DELETED
        assert watch_event.object == Example(value=42)


class TestFind(object):
    @pytest.fixture
    def client(self):
        with mock.patch.object(Example, "_client") as m:
            yield m

    def test_find_by_name(self, client):
        Example.find("app_name")
        client.get.assert_called_once_with("/example", params={"labelSelector": "app=app_name"})

    @pytest.mark.parametrize("value, selector", (
        (Equality("my_value"), "my_key=my_value"),
        (Inequality("my_value"), "my_key!=my_value"),
        (In(("value1", "value2")), "my_key in (value1,value2)"),
        (NotIn(("value1", "value2")), "my_key notin (value1,value2)"),
        (Exists(), "my_key"),
        ("my_unwrapped_value", "my_key=my_unwrapped_value"),
    ))
    def test_find_by_selectors(self, client, value, selector):
        Example.find(labels={"my_key": value})
        client.get.assert_called_once_with("/example", params={"labelSelector": selector})

    def test_repeated_keys_in_label_selector(self, client):
        labels = [
            ("foo", Inequality("bar")),
            ("foo", Exists())
        ]
        Example.find(labels=labels)

        expected_selector = "foo!=bar,foo"

        client.get.assert_called_once_with("/example", params={"labelSelector": expected_selector})


class TestDeleteList(object):

    @pytest.fixture
    def client(self):
        with mock.patch.object(Example, "_client") as m:
            yield m

    def test_delete_list(self, client):
        Example.delete_list(labels={"foo": Equality("bar"), "cat": Inequality("dog")})

        client.delete.assert_called_once_with("/example", body=None, params={"labelSelector": "cat!=dog,foo=bar"})

    def test_delete_with_options(self, client):
        opts = DeleteOptions(
            apiVersion="foo/v1",
            dryRun=[],
            gracePeriodSeconds=30,
            preconditions=Preconditions(uid="1234", resourceVersion="12"),
            propagationPolicy="Foreground"
        )
        Example.delete_list(labels={"foo": "bar"}, delete_options=opts)

        expected_body = {
            "apiVersion": "foo/v1",
            "dryRun": [],
            "gracePeriodSeconds": 30,
            "preconditions": {
                "uid": "1234",
                "resourceVersion": "12"
            },
            "propagationPolicy": "Foreground"
        }
        client.delete.assert_called_once_with("/example", params={"labelSelector": "foo=bar"}, body=expected_body)